/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* Coyote - Advanced Pilot Training Aircraft
 * Plane designed for the AIAA 2017-2018 aircraft design competition
 * by Raphael Dubois, Thibault Laurent, Bao Long Le Van, Nayan Levaux,
 *    Guillaume Noiset, Axel Piret, Arthur Scheffer, and Vincent Schmitz
 * CAD created by Nayan Levaux and exported by Adrien Crovato
 * Valid mesh obtained with gmsh 4.4.1
 */
 
/* Constants */
scl = 2; // scaling factor for lifting surfaces mesh size (testing purpose)
DefineConstant[ msFar = { 5, Name "farfield mesh size" } ];
DefineConstant[ msFus = { 0.1, Name "fuselage mesh size" } ];
DefineConstant[ msWing = { 0.036 * scl, Name "wing mesh size" } ];
DefineConstant[ msTail = { 0.028 * scl, Name "tail mesh size" } ];

/* Geometry */

Geometry.OCCAutoFix = 0;
Geometry.OCCFixDegenerated = 1;
Geometry.OCCFixSmallEdges = 1;
Geometry.OCCFixSmallFaces = 1;
Geometry.OCCSewFaces = 1;
Geometry.OCCScaling = 0.001;
Geometry.Tolerance = 1e-4; // 0.1 mm
Merge "coyote.stp";
Coherence;
Surface Loop(1) = {1:58, 65:70};
Volume(1) = {1};

// Embedded
Line{159} In Surface{68};
Line{153} In Surface{68};
Line{162} In Surface{68};
Line{168} In Surface{68};
Line{172} In Surface{68};
Line{177} In Surface{68};
Surface{59} In Volume{1};
Surface{60} In Volume{1};
Surface{61} In Volume{1};
Surface{62} In Volume{1};
Surface{63} In Volume{1};
Surface{64} In Volume{1};
Coherence;

/* Physical */

// Tips
Physical Line("wakeExW") = {156:158, 160, 165:167, 169};
Physical Line("teW") = {155, 161, 164, 170};
Physical Line("wakeTipW") = {160, 169};
Physical Line("wakeExT") = {173, 175, 176, 178, 180, 181};
Physical Line("teT") = {174, 179};
Physical Line("wakeTipT") = {173, 178};

// Downstream
Physical Surface("downstream") = {68};

// Farfield
Physical Surface("upstream") = {65};
Physical Surface("farfield") = {66, 67, 69, 70};

// Fuselage
Physical Surface("fuselage") = {1:15, 30:44};

// Wing
Physical Surface("wing") = {24, 26, 53, 55};
Physical Surface("wing_") = {25, 54};

// LEX
Physical Surface("wing") += {16, 19:23, 45, 48:52};
Physical Surface("wing_") += {17:18, 46:47};

// Tail
Physical Surface("tail") = {27, 28, 56, 57};
Physical Surface("tail_") = {29, 58};

// Wake
Physical Surface("wakeW") = {59:62};
Physical Surface("wakeT") = {63, 64};

// Internal Field
Physical Volume("field") = {1};

/* Mesh */

// Sizes
Characteristic Length {125, 122, 124, 123, 127, 129, 126, 128, 118, 110, 113, 114, 105, 97, 98, 103} = msFar; // Farfield
Characteristic Length {107, 108, 101, 100, 9, 31, 70, 2, 3, 1, 78, 65, 79, 80, 81, 82, 64, 83, 16, 17, 15, 14, 13, 55, 12, 11, 69, 56, 32, 40, 10, 29, 68, 67, 66, 30, 5, 26} = msFus; // Fuselage
Characteristic Length {4, 18, 27, 28, 54} = 0.5 * msFus; // Fuselage nose
Characteristic Length {59, 74, 37, 41, 60:63, 33:36, 38, 39, 57, 58} = 2.5 * msFus; // Intake
Characteristic Length {72, 121, 73, 24, 71, 109, 20, 112, 23, 117, 19, 96, 25, 102, 21, 22} = 2 * msFus; // Nozzle
Characteristic Length {93, 86, 91, 90, 88, 89, 87, 51, 44, 49, 48, 46, 47, 45, 106, 99} = msWing; // Wing root
Characteristic Length {43, 85} = 0.5 * msWing; // Wing root LE
Characteristic Length {93, 51, 111, 104} = 0.75 * msWing; // Wing tip
Characteristic Length {92, 50} = 0.35 * msWing; // Wing tip LE
Characteristic Length {6, 8, 42, 75, 76, 77, 116, 120} = msTail; // Tail root
Characteristic Length {7, 84} = 0.5 * msTail; // Tail root LE
Characteristic Length {52, 94, 115, 119} = 0.75 * msTail; // Tail tip TE
Characteristic Length {53, 95} = 0.35 * msTail; // Tail tip LE

// 2D
Mesh.Algorithm = 1;
// 3D
Mesh.Algorithm3D = 2;
Mesh.OptimizeNetgen = 0;
Mesh.Optimize = 1;
Mesh.Smoothing = 10;
Mesh.SmoothNormals = 1;

Mesh 1;
Geometry.Tolerance = 1e-6;
Coherence Mesh;
//Mesh 2;
//Mesh 3;
//OptimizeMesh "Gmsh"; // re-optimize just to be sure
