#!/usr/bin/env python3
# -*- coding: utf-8; -*-

# Copyright 2020 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Compute flow around the Agard 445 wing at 1 degrees AOA and Mach 0.80
# Adrien Crovato

import dart.utils as floU
import dart.default as floD
import tbox.utils as tbxU
import fwk
from fwk.testing import *
from fwk.coloring import ccolors
import numpy as np

try:
   import tboxVtk
   BestWriter = tboxVtk.VtkExport
   canPost = True
except:
   BestWriter = tbox.GmshExport
   canPost = False

def main():
	# timer
    tms = fwk.Timers()
    tms['total'].start()

    # define flow variables
    alpha = 1*np.pi/180
    M_inf = 0.8
    dim = 3

    # define dimension and mesh size
    lgt = 7. # box length
    hgt = 6. # box height
    wdt = 3. # box width
    xO = -3. # box x-origin
    zO = -3. # box z-origin
    S_ref = .35 # reference area
    c_ref = .47 # reference chord (MAC)
    rlems = 0.0028 # root leading edge mesh size
    rtems = 0.0056 # root trailing edge mesh size
    tlems = 0.0018 # tip leading mesh size
    ttems = 0.0036 # tip trailing mesh size
    fms = 1.0 # farfield mesh size

    # mesh the wing
    print(ccolors.ANSI_BLUE + 'PyMeshing...' + ccolors.ANSI_RESET)
    tms['msh'].start()
    pars = {'xL' : lgt, 'yL' : wdt, 'zL' : hgt, 'xO' : xO, 'zO' : zO, 'msLeRt' : rlems, 'msTeRt' : rtems, 'msLeTp' : tlems, 'msTeTp' : ttems, 'msF' : fms}
    msh, _ = floD.mesh(dim, 'models/agard445.geo', pars, ['field', 'wing', 'symmetry', 'downstream'], wktp = 'wakeTip')
    writer = BestWriter(msh)
    tms['msh'].stop()

    # set the problem and add fluid, initial/boundary conditions
    tms['pre'].start()
    pbl, _, _, _, _ = floD.problem(msh, dim, alpha, 0., M_inf, S_ref, c_ref, 0., 0., 0., 'wing', tp = 'wakeTip')
    tms['pre'].stop()

    # solve problem
    print(ccolors.ANSI_BLUE + 'PySolving...' + ccolors.ANSI_RESET)
    tms['solver'].start()
    solver = floD.newton(pbl)
    solver.run()
    solver.save(writer)
    tms['solver'].stop()

    # post process
    tms['post'].start()
    if canPost:
        floU.write_slices(msh.name,[0.01, 0.37, 0.75], 5)
        data = np.loadtxt(f'{msh.name}_slice_1.dat', delimiter=',', skiprows=2)
        floD.plot(data[:,0], data[:,-1], {'xlabel': 'x/c', 'ylabel': 'Cp', 'title': 'Pressure distribution at MAC', 'invert': True})
    tms['post'].stop()

    # display timers
    tms['total'].stop()
    print(ccolors.ANSI_BLUE + 'PyTiming...' + ccolors.ANSI_RESET)
    print('CPU statistics')
    print(tms)

    # check results
    print(ccolors.ANSI_BLUE + 'PyTesting...' + ccolors.ANSI_RESET)
    tests = CTests()
    tests.add(CTest('CL', solver.Cl, 0.061, 1e-1))
    tests.add(CTest('CD', solver.Cd, 0.0007, 1e-1))
    tests.add(CTest('CS', solver.Cs, 0.0017, 1e-1))
    tests.add(CTest('CM', solver.Cm, -0.064, 1e-1))
    tests.run()

    # eof
    print('')

if __name__ == "__main__":
    main()
